
#include <cpctelera.h>
#include "game.h"
#include "definitions.h"
#include "characters/characters.h"
#include "scenes/maps.h"
#include "scenes/endgame.h"
#include "sprites/shield.h"
#include "sprites/shieldiz.h"

// ======================== Transparency table ======================== //
cpctm_createTransparentMaskTable (g_tabletrans, 0x0F00, M0, 0); // Pink as transparent

// ==================================================================== //

void switchBuffers() {
	/*u8* aux 		= g_backBuffer;
	g_backBuffer 	= g_screenMem;
	g_screenMem 	= aux;

	cpct_setVideoMemoryPage( ((u16)g_screenMem) >> 10);*/

	__asm

	ld bc, (_g_backBuffer)
	ld hl, (_g_screenMem)
	ld (_g_backBuffer), hl
	ld (_g_screenMem), bc

	ld l, b
	srl l
	srl l

	call cpct_setVideoMemoryPage_asm
	__endasm;

}

void iniGame() {

	AMin[0] = 1;
	AMin[1] = 1;
	AMin[2] = 1;
	AMin[3] = 1;

	iniMap1();
}

void ejecutarEstadosPlayer() {
	TCharacter* aux;

	switch(player.estado) {
		case ANDAR:
			andar();
			break;
		case QUIETO:
			quieto();
			break;
		case DISPARAR:
			disparar(&player);
			break;
		case MINAR:
			minar();
			break;
		case BEBER:
			beber();
			break;
		case ABRIR:
			abrir();
			break;
		case ESCALERAS:
			escaleras();
			break;
		case DEFENDER:
			defender();
			break;
	}

	// mover bala
	if(player.bala.animation == 1) {
		if(player.bala.facing == RIGHT) { 
			if(player.bala.x[0] < SCREENLIMIT) {
				++player.bala.x[0];
			}
			else {
				player.bala.animation = 0;
			}
		}
		else {
			if(player.bala.x[0] > 0) {
				--player.bala.x[0];
			}
			else {
				player.bala.animation = 0;
				
			}
		}
		player.bala.draw = 2;

		// Check if enemy was hit
		aux = enemies;
		while(aux != enemies + 6) {
			if(aux->sprite.mapN == player.sprite.mapN)
				if ((player.bala.x[0] <= (aux->sprite.x[0] + 8)) &&
					(player.bala.x[0] >= (aux->sprite.x[0] - 4)) &&
					(player.bala.y[0] <= (aux->sprite.y[0] + 32)) &&
					(player.bala.y[0] >= (aux->sprite.y[0])))
				{
					--aux->hp;
					player.bala.animation = 0;
					player.bala.draw = 2;
					if(aux->hp == 0) {
						eraseSprite(&aux->sprite);
						aux->sprite.draw = 2;
						aux->sprite.mapN = 0;
					}
				}
			++aux;
		}
	}
}

void ejecutarEstadosEnemigos(TCharacter* chr) {

	switch(chr->estado) {
		case PATRULLAR:
			patrullar(chr);
			break;
		case ATACAR:
			atacar(chr);
			break;
		case ANDAR:
			break;
		case QUIETO:
			break;
		case DISPARAR:
			disparar(chr);
			break;
		case BEBER:
			break;
		case ABRIR:
			break;
	}


	
}

void moveShot(TCharacter *chr) {
	// mover bala
	if(chr->bala.animation == 1) {
		if(chr->bala.facing == RIGHT) { 
			if(chr->bala.x[0] < SCREENLIMIT) {
				++chr->bala.x[0];
				chr->bala.draw = 2;
			}
			else {
				chr->bala.animation = 0;
				chr->bala.draw = 2;
			}
		}
		else {
			if(chr->bala.x[0] > 0) {
				--chr->bala.x[0];
				chr->bala.draw = 2;
			}
			else {
				chr->bala.animation = 0;
				chr->bala.draw = 2;
			}
		}
		if (chr->bala.mapN != player.sprite.mapN) chr->bala.draw = 0;
		// Check if the player was hit
		if ((chr->bala.x[0] <= (player.sprite.x[0] + 8)) &&
			(chr->bala.x[0] >= (player.sprite.x[0] - 4)) &&
			(chr->bala.y[0] <= (player.sprite.y[0] + 32)) &&
			(chr->bala.y[0] >= (player.sprite.y[0])) && chr->bala.mapN == player.sprite.mapN)
		{
			if((chr->bala.facing == RIGHT && player.sprite.facing == LEFT && player.sprite.sprite == g_shieldiz) || (chr->bala.facing == LEFT && player.sprite.facing == RIGHT && player.sprite.sprite == g_shield)) {
				chr->bala.animation = 0;
				chr->bala.draw = 2;
			}
			else {
				--player.hp;
				chr->bala.animation = 0;
				chr->bala.draw = 2;
				if (player.hp == 0) {
					perder();
					final = 1;
				}
				map.drawHUD = 2;
			}
		}
	}
}

// ======================== Main function ============================= //
void startGame() {
	TCharacter* aux;

	iniGame();

	while(!final) {

		if(map.draw)
		{
			drawMap();
		}
		if(map.drawHUD) {
			drawHUD();
		}

		drawEntities();
		drawEnemies();

		if(player.sprite.draw) {
			reDrawSprite(&player.sprite);
		}

		if(player.bala.animation && player.bala.draw) {
			reDrawSprite(&player.bala);
		}
		else if(!player.bala.animation && player.bala.draw) {
			eraseSprite(&player.bala);
			--player.bala.draw;
		}
		aux = enemies;
		while(aux != enemies + 6) {
			if(aux->hp == 0 && aux->sprite.draw) {
				eraseSprite(&aux->sprite);
				--aux->sprite.draw;
			}
			++aux;
		}
		cpct_waitVSYNC();
		switchBuffers();

		// Player Turn
		ejecutarEstadosPlayer();

		// Enemies turn
		aux = enemies;
		while(aux != enemies + 6) {
			if(aux->sprite.mapN == player.sprite.mapN)
				ejecutarEstadosEnemigos(aux);
			moveShot(aux);
			++aux;
		}
	}
}

// ========================= Draw statics ============================= //




